@@ -7,6 +7,7 @@
  * Copyright (c) 2001, 2002 Free Software Foundation
  */
 
+#include <strings.h>
 #include <config.h>
 #include <stdio.h>
 #include <gtk/gtksignal.h>
@@ -18,6 +19,7 @@
 
 #include <ggv-postscript-view.h>
 #include <ggv-control.h>
+#include <ggv-document.h>
 
 #include <gtkgs.h>
 #include <ps.h>
@@ -70,7 +72,6 @@ enum {
 	PROP_DEFAULT_SIZE,
 	PROP_OVERRIDE_SIZE,
 	PROP_RESPECT_EOF,
-	PROP_WATCH_FILE,
 	PROP_WIDTH,
 	PROP_HEIGHT,
 	PROP_ORIENTATION,
@@ -218,12 +219,12 @@ notify_page_count_change(GgvPostScriptView *ps_view)
 	BonoboArg *arg;
 
 	set_page_items_sensitivity(ps_view);
-	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_page_count(ps_view) > 0);
+	set_file_items_sensitivity(ps_view, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	if(ps_view->priv->sidebar)
 		ggv_sidebar_create_page_list(ps_view->priv->sidebar);
 
 	arg = bonobo_arg_new(TC_CORBA_long);
-	BONOBO_ARG_SET_LONG(arg, ggv_postscript_view_get_page_count(ps_view));
+	BONOBO_ARG_SET_LONG(arg, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)));
 	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
 										 "Bonobo/Property:change:page_count",
 										 arg, NULL);
@@ -306,9 +307,10 @@ ggv_postscript_view_get_current_page(GgvPostScriptView *ps_view)
 	return gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
 }
 
-gint
-ggv_postscript_view_get_page_count(GgvPostScriptView *ps_view)
+static gint
+ggv_postscript_view_get_page_count(GgvDocument *doc)
 {
+    GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(doc);
 	return gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
 }
 
@@ -322,7 +324,7 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	ggv_postscript_view_goto_page(ps_view, 0);
 	notify_page_change(ps_view);
 	ggv_sidebar_update_coordinates(ps_view->priv->sidebar, 0.0, 0.0);
-	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_page_count(ps_view) > 0);
+	set_file_items_sensitivity(ps_view, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	notify_status_change(ps_view);
 }
 
@@ -379,9 +381,12 @@ load_ps_from_stream (BonoboPersistStream *ps,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
+	if(ps_view->priv->uri) {
+		g_free(ps_view->priv->uri);
+		ps_view->priv->uri = NULL;
+	}
 	ps_view_clean_tmp_file(ps_view);
 	tmpfile = ps_view_get_tmp_file(ps_view);
-
 	do {
 		Bonobo_Stream_read (stream, 32768, &buffer, ev);
 		if (ev->_major != CORBA_NO_EXCEPTION)
@@ -528,6 +533,12 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 
 	ps_view_clean_tmp_file(ps_view);
 
+	uri_str = ps_view->priv->uri;
+	ps_view->priv->uri = g_strdup(filename);
+	if(NULL != uri_str) {
+		g_free(uri_str);
+		uri_str = NULL;
+	}
 	uri = gnome_vfs_uri_new(filename);
 	if(gnome_vfs_uri_is_local(uri)) {
 		uri_str = gnome_vfs_uri_to_string(uri, GNOME_VFS_URI_HIDE_NONE);
@@ -821,7 +832,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 										 ps_view->priv->save_path);
 		if(fname) {
 			if((local_path = gnome_vfs_get_local_path_from_uri(fname)) &&
-			   ggv_file_readable(local_path)) {
+			   ggv_file_length(local_path) >= 0) {
 				GtkWidget *dlg;
 		
 				g_free(local_path);
@@ -1020,19 +1031,21 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 										 orientation_paths[i] + strlen("/commands/"),
 										 listener_Orientation_cb, ps_view);
 	set_file_items_sensitivity(ps_view,
-							   ggv_postscript_view_get_page_count(ps_view) > 0);
+							   ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	set_page_items_sensitivity(ps_view);
 	sync_orientation_items(ps_view);
 	sync_size_items(ps_view);
 }
 
-gchar **
-ggv_postscript_view_get_page_names(GgvPostScriptView *ps_view)
+static gchar **
+ggv_postscript_view_get_page_names(GgvDocument *doc)
 {
 	gchar **names = NULL;
 	GtkGS *gs;
 	gint i;
+	GgvPostScriptView *ps_view;
 
+	ps_view = GGV_POSTSCRIPT_VIEW(doc);
 	gs = GTK_GS(ps_view->priv->gs);
 	if(gs->loaded && gs->doc && gs->structured_doc) {
 		names = g_new0(gchar *, gs->doc->numpages + 1);
@@ -1225,14 +1238,6 @@ ggv_postscript_view_get_prop(BonoboPropertyBag *bag,
 			CORBA_TRUE:CORBA_FALSE;
 		break;
 	}
-	case PROP_WATCH_FILE: {
-		g_assert(arg->_type == TC_CORBA_boolean);
-
-		*(CORBA_boolean *)arg->_value =
-			gtk_gs_get_watch_file(GTK_GS(ps_view->priv->gs))?
-			CORBA_TRUE:CORBA_FALSE;
-		break;
-	}
 	default:
 		g_assert_not_reached();
 	}
@@ -1327,11 +1332,6 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 								*(CORBA_boolean *)arg->_value);
 		 break;
 	}
-	case PROP_WATCH_FILE: {
-		 gtk_gs_set_watch_file(GTK_GS(ps_view->priv->gs),
-							   *(CORBA_boolean *)arg->_value);
-		 break;
-	}
 	default:
 		g_assert_not_reached();
 	}
@@ -1351,28 +1351,29 @@ ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 	g_return_val_if_fail(ps_view != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), CORBA_OBJECT_NIL);
 
+#ifdef _DEBUG
 	g_message ("ggv_postscript_view_get_object: %d - %s",
 			   only_if_exists, item_name);
-
+#endif /* _DEBUG */
 	params = ggv_split_string (item_name, "!");
 	for (c = params; c; c = c->next) {
 		gchar *name = c->data;
 
 		if ((!strcmp (name, "control") || !strcmp (name, "embeddable"))
 		    && (object != NULL)) {
-			g_message ("ggv_postscript_view_get_object: "
+			g_warning ("ggv_postscript_view_get_object: "
 					   "can only return one kind of an Object");
 			continue;
 		}
 
 		if (!strcmp (name, "control"))
 			object = (BonoboObject *) ggv_control_new (ps_view);
-#if 0
+#ifdef ENABLE_GGV_EMBEDDABLE
 		else if (!strcmp (item_name, "embeddable"))
 			object = (BonoboObject *) ggv_embeddable_new (image);
-#endif
+#endif /* ENABLE_GGV_EMBEDDABLE */
 		else
-			g_message ("ggv_postscript_view_get_object: "
+			g_warning ("ggv_postscript_view_get_object: "
 					   "unknown parameter `%s'",
 					   name);
 	}
@@ -1627,10 +1628,10 @@ ggv_postscript_view_finalize(GObject *object)
 
 	ps_view_clean_tmp_file(ps_view);
 
-	if(ps_view->priv->save_path) {
+	if(ps_view->priv->save_path)
 		g_free(ps_view->priv->save_path);
-		ps_view->priv->save_path = NULL;
-	}
+	if(ps_view->priv->uri)
+		g_free(ps_view->priv->uri);
 
 	g_free(ps_view->priv);
 
@@ -1680,7 +1681,7 @@ impl_GNOME_GGV_PostScriptView_getPages(PortableServer_Servant servant,
 
 static void
 impl_GNOME_GGV_PostScriptView_reload(PortableServer_Servant servant,
-									   CORBA_Environment *ev)
+									 CORBA_Environment *ev)
 {
 	GgvPostScriptView *ps_view;
 
@@ -1729,16 +1730,54 @@ ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 	ggv_prefs_load();
 }
 
+static void
+ggv_document_interface_init (GgvDocumentClass *klass, gpointer iface_data)
+{
+	klass->get_page_count = ggv_postscript_view_get_page_count;
+	klass->get_page_names = ggv_postscript_view_get_page_names;
+}
+
 static void
 ggv_postscript_view_init(GgvPostScriptView *ps_view)
 {
 	ps_view->priv = g_new0(GgvPostScriptViewPrivate, 1);
 }
 
-BONOBO_TYPE_FUNC_FULL(GgvPostScriptView, 
-					  GNOME_GGV_PostScriptView,
-					  BONOBO_TYPE_OBJECT,
-					  ggv_postscript_view);
+GType
+ggv_postscript_view_get_type (void)
+{
+    static GType type = 0;
+    
+    if (type == 0) {
+		static GTypeInfo info = {
+			sizeof (GgvPostScriptViewClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) ggv_postscript_view_class_init,
+			NULL, NULL,
+			sizeof (GgvPostScriptView),
+			0,
+			(GInstanceInitFunc) ggv_postscript_view_init
+		};
+		
+		static const GInterfaceInfo ggv_document_info = {
+			(GInterfaceInitFunc) ggv_document_interface_init,
+			NULL,
+			NULL
+		};
+		
+		type = bonobo_type_unique(BONOBO_TYPE_OBJECT,
+								  POA_GNOME_GGV_PostScriptView__init,
+								  POA_GNOME_GGV_PostScriptView__fini,
+								  G_STRUCT_OFFSET (GgvPostScriptViewClass, epv),
+								  &info, "GgvPostScriptView");
+		
+		g_type_add_interface_static (type,
+									 GGV_DOCUMENT_TYPE,
+									 &ggv_document_info);
+    }
+    return type;
+}
 
 #if 0
 static void
@@ -1795,24 +1834,14 @@ property_control_get_cb(BonoboPropertyControl *property_control,
 #endif
 }
 
-static void
-document_changed_cb(GtkGS *gs, gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
-
-	gtk_gs_reload(gs);
-	notify_page_count_change(GGV_POSTSCRIPT_VIEW(data));
-	notify_page_change(GGV_POSTSCRIPT_VIEW(data));
-	notify_status_change(GGV_POSTSCRIPT_VIEW(data));
-}
-
 static void
 interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 
 #if 0
-	/* actually, these are quite useless, so we won't show them... */
+	/* actually, these are quite useless, so we don't want to show them,
+	   hence the '#if 0'... */
 	if(ps_view->priv->msg_win)
 		ggv_msg_window_add_text(ps_view->priv->msg_win, msg, TRUE);
 #endif
@@ -1859,11 +1888,6 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 		gtk_gs_set_default_orientation(gs, gconf_value_get_int(entry->value));
 		sync_orientation_items(ps_view);
 	}
-#if 0
-	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/zoom")) {
-		gtk_gs_set_zoom(gs, gconf_value_get_float(entry->value));
-	}
-#endif
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/size")) {
 		gtk_gs_set_default_size(gs, gconf_value_get_int(entry->value));
 		sync_size_items(ps_view);
@@ -1874,8 +1898,6 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 		gtk_gs_set_override_size(gs, gconf_value_get_bool(entry->value));
 		sync_size_items(ps_view);
 	}
-	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/watch_doc"))
-		gtk_gs_set_watch_file(gs, gconf_value_get_bool(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/scrollstep"))
 		gtk_gs_set_scroll_step(gs, gconf_value_get_float(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/show_scroll_rect"))
@@ -1884,6 +1906,8 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 	   ps_view->priv->pane_auto_jump = gconf_value_get_bool(entry->value);
 	else if(!strcmp(entry->key, "/apps/ggv/control/pageflip"))
 	   ps_view->priv->page_flip = gconf_value_get_bool(entry->value);
+	/* note that we don't want to handle changes to some parameters
+	   that are considered "default": zoom, etc. */
 }
 
 static gboolean
@@ -1926,6 +1950,13 @@ sidebar_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	return TRUE;
 }
 
+static void
+sidebar_set_page(GgvSidebar *sidebar, int page, gpointer data)
+{
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+	ggv_postscript_view_goto_page(ps_view, page);
+}
+
 GgvPostScriptView *
 ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							  GtkGS *gs, gboolean zoom_fit)
@@ -1965,8 +1996,6 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 					 G_CALLBACK(interpreter_message_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "interpreter_error",
 					 G_CALLBACK(interpreter_error_cb), ps_view);
-	g_signal_connect(G_OBJECT(gs), "document_changed",
-					 G_CALLBACK(document_changed_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "realize",
 					 G_CALLBACK(view_realized_cb), NULL);
 
@@ -2032,10 +2061,6 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							PROP_RESPECT_EOF, TC_CORBA_boolean, NULL,
 							_("Respect EOF comment"), 
 							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
-	bonobo_property_bag_add(ps_view->priv->property_bag, "watch_file",
-							PROP_WATCH_FILE, TC_CORBA_boolean, NULL,
-							_("Watch displayed file for changes"), 
-							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
 	bonobo_property_bag_add(ps_view->priv->property_bag, "antialiasing",
 							PROP_ANTIALIASING, TC_CORBA_boolean, NULL,
 							_("Antialiasing"), 
@@ -2048,9 +2073,11 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 	/* UI Component */
 	ps_view->priv->uic = bonobo_ui_component_new("GgvPostScriptView");
 
-	ps_view->priv->sidebar = ggv_sidebar_new(ps_view);
+	ps_view->priv->sidebar = ggv_sidebar_new(GGV_DOCUMENT(ps_view));
 	g_signal_connect(G_OBJECT(ggv_sidebar_get_checklist(ps_view->priv->sidebar)),
 					 "key_press_event", G_CALLBACK(sidebar_key_press_event), ps_view);
+	g_signal_connect(G_OBJECT(ps_view->priv->sidebar), "set_page",
+					 G_CALLBACK(sidebar_set_page), ps_view);
 
 	ps_view->priv->pane_auto_jump =
 		gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
