@@ -31,6 +31,9 @@
 
 #define GGV_POSTSCRIPT_VIEWER_CONTROL_IID "OAFIID:GNOME_GGV_Control"
 
+#define GGV_WATCH_TIMEOUT 2
+#define GGV_WATCH_INTERVAL 1000
+
 static BonoboWindowClass *parent_class;
 
 static GList *window_list = NULL;
@@ -40,6 +43,24 @@ enum {
         TARGET_URI_LIST,
 };
 
+static void
+set_file_cmds_sensitivity(GgvWindow *win, gboolean sens)
+{
+        if(NULL == win->uic)
+                return;
+
+        bonobo_ui_component_set_prop(win->uic,
+                                     "/commands/FileReload",
+                                     "sensitive",
+                                     sens?"1":"0",
+                                     NULL);
+        bonobo_ui_component_set_prop(win->uic,
+                                     "/commands/FileClose",
+                                     "sensitive",
+                                     sens?"1":"0",
+                                     NULL);
+}
+
 static void
 sync_settings_menu_items(GgvWindow *win)
 {
@@ -134,6 +155,53 @@ sync_fullscreen_items(GgvWindow *win)
         }
 }
 
+static void
+ggv_window_remove_control(GgvWindow *win)
+{
+        CORBA_Environment ev;
+        
+        CORBA_exception_init(&ev);
+
+        if(win->ctlframe != NULL && win->control != CORBA_OBJECT_NIL) {
+                bonobo_control_frame_control_deactivate(win->ctlframe);
+                bonobo_control_frame_bind_to_control(win->ctlframe, NULL, &ev);
+                if(BONOBO_EX(&ev)) {
+                        g_warning("Could not bind to NIL: '%s'",
+                                  bonobo_exception_get_text(&ev));
+                        CORBA_exception_free(&ev);
+                        return;
+                }
+        }
+        if(win->control != CORBA_OBJECT_NIL) {
+                bonobo_object_release_unref(win->control, NULL);
+                win->control = CORBA_OBJECT_NIL;
+        }
+        if(win->popup_uic) {
+                bonobo_object_unref(win->popup_uic);
+                win->popup_uic = NULL;
+        }
+        if(win->pb) {
+                bonobo_object_release_unref(win->pb, NULL);
+                win->pb = CORBA_OBJECT_NIL;
+        }
+        if(win->view) {
+                gtk_container_remove(GTK_CONTAINER(win->hbox),
+                                     win->view);
+                gtk_widget_unref(win->view);
+                win->view = NULL;
+        }
+        if(win->filename) {
+                g_free(win->filename);
+                win->filename = NULL;
+        }
+        if(0 != win->timer_tag) {
+                gtk_timeout_remove(win->timer_tag);
+                win->timer_tag = 0;
+        }
+        set_file_cmds_sensitivity(win, FALSE);
+        CORBA_exception_free(&ev);                
+}
+
 /* utility functions to raise window above all others! */
 #define XA_WIN_LAYER         "_WIN_LAYER"
 typedef enum {
@@ -336,24 +404,22 @@ control_property_changed_handler(BonoboListener    *listener,
                         if(arg == NULL)
                                 return;
                         msg = BONOBO_ARG_GET_STRING(arg);
-                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
-                                                     GTK_DIALOG_MODAL,
-                                                     GTK_MESSAGE_ERROR,
-                                                     GTK_BUTTONS_OK,
-                                                     msg);
-                        gtk_widget_show(dlg);
-                        gtk_dialog_run(GTK_DIALOG(dlg));
-                        gtk_widget_destroy(dlg);
+                        if(*msg != '\0') {
+                                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                             GTK_DIALOG_MODAL,
+                                                             GTK_MESSAGE_ERROR,
+                                                             GTK_BUTTONS_OK,
+                                                             msg);
+                                gtk_widget_show(dlg);
+                                gtk_dialog_run(GTK_DIALOG(dlg));
+                                gtk_widget_destroy(dlg);
+                        }
                         bonobo_arg_release(arg);
                         return;
                 }
         }
         else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
-                bonobo_ui_component_set_prop(win->uic,
-                                             "/commands/FileReload",
-                                             "sensitive",
-                                             (BONOBO_ARG_GET_LONG(any) > 0)?"1":"0",
-                                             NULL);
+                set_file_cmds_sensitivity(win, (BONOBO_ARG_GET_LONG(any) > 0));
         }
 }
 
@@ -408,7 +474,11 @@ verb_FileClose(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
 
-        ggv_window_close(win);
+        if(window_list->next != NULL)
+                ggv_window_close(win);
+        else {
+                ggv_window_remove_control(win);
+        }
 }
 
 static void
@@ -520,8 +590,9 @@ static void
 verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	static GtkWidget *about = NULL;
+	static gboolean translated = FALSE;
 	static const char *authors[] = {
-                "Jaka Mocnik (current maintainer)",
+                N_("Jaka Mocnik (current maintainer)"),
                 "Jonathan Blandford",
                 "Daniel M. German",
                 "Satyajit Kanungo",
@@ -531,19 +602,24 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
                 "Johannes Plass",
 		"Istvan Szekeres",
                 "Tim Theisen",
-                "And many more...",
+                N_("And many more..."),
  		NULL
 	};
-
 	static const char *documenters[] = {
+		"Sun Microsystems <gdocteam@sun.com>",
 		NULL
 	};
-
-	/* Translator Credits */
 	char *translator_credits = _("translator_credits");
-
         static GdkPixbuf *logo = NULL;
 
+	if (!translated) {
+		int i;
+
+		for (i = 0; authors[i] != NULL; i++)
+		    authors[i] = _(authors[i]);
+		translated = TRUE;
+	}
+
         if(!logo)
                 logo = gdk_pixbuf_new_from_file(GNOMEICONDIR
                                                 "/ggv/ggv-splash.png",
@@ -552,7 +628,7 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 		about = gnome_about_new (
 			_("Gnome Ghostview"),
 			VERSION,
-			_("Copyright (C) 1998-2002 The Free Software Foundation"),
+			"Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
 			_("The GNOME PostScript document previewer"),
 			authors,
 			documenters,
@@ -662,36 +738,18 @@ static void
 ggv_window_destroy(GtkObject *object)
 {
         GgvWindow *win;
+        gint i;
 
         g_return_if_fail(object != NULL);
         g_return_if_fail(GGV_IS_WINDOW(object));
 
         win = GGV_WINDOW(object);
 
-        if(win->ctlframe != NULL) {
-                bonobo_object_unref(BONOBO_OBJECT(win->ctlframe));
-                win->ctlframe = NULL;
-        }
-        if(win->control != CORBA_OBJECT_NIL) {
-                bonobo_object_release_unref(win->control, NULL);
-                win->control = CORBA_OBJECT_NIL;
-        }
         if(win->uic) {
                 bonobo_object_unref(win->uic);
                 win->uic = NULL;
         }
-        if(win->popup_uic) {
-                bonobo_object_unref(win->popup_uic);
-                win->popup_uic = NULL;
-        }
-        if(win->pb) {
-                bonobo_object_release_unref(win->pb, NULL);
-                win->pb = CORBA_OBJECT_NIL;
-        }
-        if(win->filename) {
-                g_free(win->filename);
-                win->filename = NULL;
-        }
+        ggv_window_remove_control(win);
 
         window_list = g_list_remove(window_list, win);
 
@@ -792,6 +850,7 @@ static void
 ggv_window_init (GgvWindow *window)
 {
         window_list = g_list_prepend(window_list, window);
+        window->watch_doc = FALSE;
 }
 
 GType
@@ -826,29 +885,14 @@ ggv_window_new(void)
 {
         GgvWindow *win;
 	BonoboUIContainer *ui_container;
-	BonoboUIComponent *uic, *popup_uic;
+	BonoboUIComponent *uic;
         CORBA_Environment ev;
-        Bonobo_Control control;
-        GtkWidget *widget;
-        Bonobo_PropertyBag pb;
-        gchar *mask;
 
 	static const GtkTargetEntry drag_types[] = {
 		{ "text/uri-list", 0, TARGET_URI_LIST }
 	};
 
-        /* get the control */
-        CORBA_exception_init(&ev);
-        control = bonobo_get_object(GGV_POSTSCRIPT_VIEWER_CONTROL_IID,
-                                    "Bonobo/Control", &ev);
-        if(BONOBO_EX(&ev) || control == CORBA_OBJECT_NIL) {
-                CORBA_exception_free(&ev);
-                return NULL;
-        }
-        CORBA_exception_free(&ev);
-
 	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("GGV: no document loaded"), NULL));
-        win->control = control;
 
         win->current_page = -2;
         win->show_toolbar = ggv_toolbar;
@@ -889,7 +933,7 @@ ggv_window_new(void)
         win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                           NULL);
-	bonobo_ui_util_set_ui(uic, NULL, "ggv-ui.xml", "GGV", NULL);
+	bonobo_ui_util_set_ui(uic, DATADIR, "ggv-ui.xml", "GGV", NULL);
         sync_toolbar_style(win);
 	bonobo_ui_component_add_verb_list_with_data(uic, ggv_app_verbs, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowMenus",
@@ -914,59 +958,6 @@ ggv_window_new(void)
 	bonobo_control_frame_set_autoactivate(win->ctlframe, FALSE);
 	g_signal_connect(G_OBJECT(win->ctlframe), "activate_uri",
                          (GtkSignalFunc)control_frame_activate_uri, NULL);
-	bonobo_control_frame_bind_to_control(win->ctlframe, control, NULL);
-	widget = bonobo_control_frame_get_widget(win->ctlframe);
-	gtk_widget_show(widget);
-        gtk_box_pack_start(GTK_BOX(win->hbox), widget,
-                           TRUE, TRUE, 0);
-	bonobo_control_frame_control_activate(win->ctlframe);
-
-        /* now get the control's property bag */
-        pb = bonobo_control_frame_get_control_property_bag(win->ctlframe, NULL);
-        if(pb == CORBA_OBJECT_NIL) {
-                g_warning("Control does not have any properties.");
-        }
-        else {
-                /* TODO: set initial status & title */
-                mask =  "Bonobo/Property:change:page,"
-                        "Bonobo/Property:change:page_count,"
-                        "Bonobo/Property:change:width,"
-                        "Bonobo/Property:change:height,"
-                        "Bonobo/Property:change:title,"
-                        "Bonobo/Property:change:status";
-                bonobo_ui_component_set_prop(win->uic,
-                                             "/commands/FileReload",
-                                             "sensitive",
-                                             (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
-                                             NULL);
-                bonobo_event_source_client_add_listener(pb,
-                                                        (BonoboListenerCallbackFn)control_property_changed_handler,
-                                                        mask, NULL, win);
-                win->pb = pb;
-        }
-
-        /* merge our items in the control's popup menu */
-        popup_uic = bonobo_control_frame_get_popup_component(win->ctlframe, NULL);
-        if(popup_uic == NULL) {
-                g_warning("Control does not have a popup component.");
-        }
-        else {
-                bonobo_ui_util_set_ui(popup_uic, NULL, "ggv-ui.xml",
-                                      "GGV", NULL);
-                bonobo_ui_component_add_listener(popup_uic, "SettingsShowMenus",
-                                                 listener_SettingsShow, win);
-                bonobo_ui_component_add_listener(popup_uic, "SettingsShowSidebar",
-                                                 listener_SettingsShow, win);
-                bonobo_ui_component_add_listener(popup_uic, "SettingsShowToolbar",
-                                                 listener_SettingsShow, win);
-                bonobo_ui_component_add_listener(popup_uic, "SettingsShowStatusbar",
-                                                 listener_SettingsShow, win);
-                bonobo_ui_component_add_listener(popup_uic, "ViewFullscreen",
-                                                 listener_ViewFullscreen, win);
-                win->popup_uic = popup_uic;
-                sync_settings_popup_items(win);
-        }
-
 	/* set default geometry */
         gtk_widget_set_usize(GTK_WIDGET(win),
                              ggv_default_width, ggv_default_height);
@@ -981,6 +972,8 @@ ggv_window_new(void)
                                 (GConfClientNotifyFunc)prefs_changed,
                                 win, NULL, NULL);
 
+        set_file_cmds_sensitivity(win, FALSE);
+
 	return GTK_WIDGET(win);
 }
 
@@ -989,39 +982,135 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 {
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
-        gchar *title, *unescaped;
+        gchar *title, *unescaped, *utf8_title;
+	BonoboUIComponent *popup_uic;
+        Bonobo_Control control;
+        GtkWidget *widget;
+        Bonobo_PropertyBag pb;
+        gchar *mask;
+        GnomeVFSFileInfo info;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
         g_return_val_if_fail(filename != NULL, FALSE);
 
+        ggv_window_set_watch_file(win, FALSE);
+
+	if (GNOME_VFS_OK == gnome_vfs_get_file_info(filename, &info, 
+                                                    GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+                                                    GNOME_VFS_FILE_INFO_FOLLOW_LINKS)) {
+                win->mtime = info.mtime;
+        }
+        else
+                return FALSE;
+
         CORBA_exception_init(&ev);
+
+        if(NULL == win->control) {
+                control = bonobo_get_object(GGV_POSTSCRIPT_VIEWER_CONTROL_IID,
+                                            "Bonobo/Control", &ev);
+                if(BONOBO_EX(&ev) || control == CORBA_OBJECT_NIL) {
+                        g_warning("Could not get GGV control: '%s'",
+                                  bonobo_exception_get_text(&ev));
+                        CORBA_exception_free(&ev);
+                        return FALSE;
+                }
+                CORBA_exception_free(&ev);
+
+                /* insert the control */
+                win->control = control;
+                bonobo_control_frame_bind_to_control(win->ctlframe, control,
+                                                     NULL);
+                widget = bonobo_control_frame_get_widget(win->ctlframe);
+                win->view = widget;
+                gtk_widget_ref(win->view);
+                gtk_widget_show(widget);
+                gtk_box_pack_start(GTK_BOX(win->hbox), widget,
+                                   TRUE, TRUE, 0);
+                bonobo_control_frame_control_activate(win->ctlframe);
+                
+                /* now get the control's property bag */
+                pb = bonobo_control_frame_get_control_property_bag
+                        (win->ctlframe, NULL);
+                if(pb == CORBA_OBJECT_NIL) {
+                        g_warning("Control does not have any properties.");
+                }
+                else {
+                        /* TODO: set initial status & title */
+                        mask =  "Bonobo/Property:change:page,"
+                                "Bonobo/Property:change:page_count,"
+                                "Bonobo/Property:change:width,"
+                                "Bonobo/Property:change:height,"
+                                "Bonobo/Property:change:title,"
+                                "Bonobo/Property:change:status";
+                        bonobo_ui_component_set_prop
+                                (win->uic, "/commands/FileReload", "sensitive",
+                                 (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
+                                 NULL);
+                        bonobo_ui_component_set_prop
+                                (win->uic, "/commands/FileClose", "sensitive",
+                                 (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
+                                 NULL);
+                        bonobo_event_source_client_add_listener
+                                (pb,
+                                 (BonoboListenerCallbackFn)control_property_changed_handler,
+                                 mask, NULL, win);
+                        win->pb = pb;
+                }
+                
+                /* merge our items in the control's popup menu */
+                popup_uic = bonobo_control_frame_get_popup_component
+                        (win->ctlframe, NULL);
+                if(popup_uic == NULL) {
+                        g_warning("Control does not have a popup component.");
+                }
+                else {
+                        bonobo_ui_util_set_ui(popup_uic, DATADIR, "ggv-ui.xml",
+                                              "GGV", NULL);
+                        bonobo_ui_component_add_listener
+                                (popup_uic, "SettingsShowMenus",
+                                 listener_SettingsShow, win);
+                        bonobo_ui_component_add_listener
+                                (popup_uic, "SettingsShowSidebar",
+                                 listener_SettingsShow, win);
+                        bonobo_ui_component_add_listener
+                                (popup_uic, "SettingsShowToolbar",
+                                 listener_SettingsShow, win);
+                        bonobo_ui_component_add_listener
+                                (popup_uic, "SettingsShowStatusbar",
+                                 listener_SettingsShow, win);
+                        bonobo_ui_component_add_listener
+                                (popup_uic, "ViewFullscreen",
+                                 listener_ViewFullscreen, win);
+                        win->popup_uic = popup_uic;
+                        sync_settings_popup_items(win);
+                }
+        }
+                
+        /* store filename and set title */
+        if(win->filename)
+                g_free(win->filename);
+        win->filename = g_strdup(filename);
+        unescaped = gnome_vfs_unescape_string_for_display(win->filename);
+        title = g_strconcat(_("GGV: "), unescaped, NULL);
+        g_free(unescaped);
+        utf8_title = g_locale_to_utf8(title, -1, NULL, NULL, NULL);
+        g_free(title);
+        gtk_window_set_title(GTK_WINDOW(win), utf8_title);
+        g_free(utf8_title);
+        
         pf = Bonobo_Unknown_queryInterface(win->control,
                                            "IDL:Bonobo/PersistFile:1.0", &ev);
         if(BONOBO_EX(&ev) || pf == CORBA_OBJECT_NIL) {
                 CORBA_exception_free(&ev);
                 return FALSE;
         }
-
-        Bonobo_PersistFile_load(pf, filename, &ev);
+        Bonobo_PersistFile_load(pf, win->filename, &ev);
         bonobo_object_release_unref(pf, NULL);
-
-        if(win->filename)
-                g_free(win->filename);
-        win->filename = g_strdup(filename);
-
-        unescaped = gnome_vfs_unescape_string_for_display(win->filename);
-        title = g_strconcat(_("GGV: "), unescaped, NULL);
-        g_free(unescaped);
-        gtk_window_set_title(GTK_WINDOW(win), title);
-        g_free(title);
-
-        if(BONOBO_EX(&ev)) {
-                CORBA_exception_free(&ev);
-                return FALSE;
-        }
         CORBA_exception_free(&ev);
 
+        ggv_window_set_watch_file(win, ggv_watch_doc);
+
         return TRUE;
 }
 
@@ -1057,3 +1146,73 @@ ggv_get_window_list()
 {
         return window_list;
 }
+
+/* Reload file if it was changed */
+static gint
+timer_callback (gpointer data)
+{
+        GgvWindow *win;
+        GnomeVFSFileInfo info;
+	struct stat sbuf;
+
+        win = GGV_WINDOW(data);
+
+        if(NULL == win->filename)
+                return FALSE;
+
+        /* if global prefs changed, change our local prefs */
+        if(!ggv_watch_doc) {
+                ggv_window_set_watch_file(win, FALSE);
+                return FALSE;
+        }
+
+        if(GNOME_VFS_OK != gnome_vfs_get_file_info(win->filename, &info, 0))
+                return TRUE;
+
+        /* reload file if necessary */
+	if ((win->current_page >= 0) &&
+	    (info.mtime != win->mtime) &&
+	    (time (NULL) - info.mtime >= GGV_WATCH_TIMEOUT) &&
+            (NULL != win->control)) {
+                GNOME_GGV_PostScriptView ps_view;
+                CORBA_Environment ev;
+
+                CORBA_exception_init(&ev);
+                ps_view = (GNOME_GGV_PostScriptView)
+                        Bonobo_Unknown_queryInterface(win->control,
+                                                      "IDL:GNOME/GGV/PostScriptView:1.0",
+                                                      &ev);
+                if(!BONOBO_EX(&ev)) {
+                        GNOME_GGV_PostScriptView_reload(ps_view, &ev);
+                        if(BONOBO_EX(&ev))
+                                g_warning("Could not invoke reload method.");
+                        bonobo_object_release_unref(ps_view, &ev);
+                }
+                CORBA_exception_free(&ev);
+                win->mtime = info.mtime;
+        }
+
+        return TRUE;
+}
+
+void
+ggv_window_set_watch_file(GgvWindow *win, gboolean f)
+{
+        if(win->watch_doc == f)
+                return;
+
+        win->watch_doc = f;
+	if(win->watch_doc) {
+		if(win->timer_tag)
+                        gtk_timeout_remove(win->timer_tag);
+		win->timer_tag =
+                        gtk_timeout_add(GGV_WATCH_INTERVAL,
+                                        timer_callback, win);
+	}
+        else {
+		if(win->timer_tag) {
+                        gtk_timeout_remove(win->timer_tag);
+                        win->timer_tag = 0;
+                }
+        }
+}
