@@ -146,12 +146,13 @@ The DONE message indicates that ghostscript has finished processing.
 #define GTK_GS_WATCH_INTERVAL 1000
 #define GTK_GS_WATCH_TIMEOUT  2
 
-enum { INTERPRETER_MESSAGE, INTERPRETER_ERROR, DOCUMENT_CHANGED, LAST_SIGNAL };
+enum { INTERPRETER_MESSAGE, INTERPRETER_ERROR, LAST_SIGNAL };
 
 static gboolean broken_pipe = FALSE;
 
 static void catchPipe(int i)
 {
+        g_message("in catch Pipe!!!\n");
         broken_pipe = True;
 }
 
@@ -347,7 +348,6 @@ gtk_gs_init (GtkGS *gs)
         gs->antialiased = gtk_gs_defaults_get_antialiased();
         gs->override_size = gtk_gs_defaults_get_override_size();
         gs->respect_eof = gtk_gs_defaults_get_respect_eof();
-        gs->watch_doc = gtk_gs_defaults_get_watch_doc();
         gs->show_scroll_rect = gtk_gs_defaults_get_show_scroll_rect();
         gs->scroll_step = gtk_gs_defaults_get_scroll_step();
         gs->zoom_mode = gtk_gs_defaults_get_zoom_mode();
@@ -391,17 +391,6 @@ gtk_gs_class_init (GtkGSClass *klass)
                        G_TYPE_NONE,
                        1, G_TYPE_INT);
 
-	gtk_gs_signals[DOCUMENT_CHANGED] = g_signal_new(
-                       "document_changed",
-                       G_TYPE_FROM_CLASS(object_class),
-                       G_SIGNAL_RUN_LAST,
-                       G_STRUCT_OFFSET(GtkGSClass, document_changed),
-                       NULL,
-                       NULL,
-                       g_cclosure_marshal_VOID__VOID,
-                       G_TYPE_NONE,
-                       0);
-
         object_class->destroy = gtk_gs_destroy;
 
         widget_class->realize       = gtk_gs_realize;
@@ -466,10 +455,6 @@ gtk_gs_cleanup (GtkGS *gs)
                 g_free(gs->gs_filename_unc);
                 gs->gs_filename_unc = NULL;
         }
-        if(gs->timer_tag) {
- 		gtk_timeout_remove (gs->timer_tag);
- 		gs->timer_tag = 0;
- 	}
         if(gs->pstarget && !gdk_window_is_visible(gs->pstarget))
                 gdk_window_hide(gs->pstarget);
         gs->current_page = -1;
@@ -800,32 +785,6 @@ send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close)
         }
 }
 
-/* Reload file if it was changed */
-static gint
-timer_callback (gpointer data)
-{
-        GtkGS *gs;
-	struct stat sbuf;
-
-        g_assert (data);
-
-        gs = GTK_GS(data);
-
-        /* remove timer if no valid file is loaded */
-	if (!gs->loaded || !gs->watch_doc || !gs->gs_psfile) {
-                gs->timer_tag = 0;
-		return FALSE;
-	}
-
-	if (!gs->busy && !stat(gs->gs_filename, &sbuf) &&
-	    (sbuf.st_mtime != gs->mtime) &&
-	    (time (NULL) - sbuf.st_mtime >= GTK_GS_WATCH_TIMEOUT)) {
-                gtk_signal_emit(GTK_OBJECT(gs), gtk_gs_signals[DOCUMENT_CHANGED]);
-        }
-
-        return TRUE;
-}
-
 static void
 set_up_page (GtkGS *gs) 
      /* 
@@ -1407,15 +1366,17 @@ check_pdf (GtkGS *gs)
                 }
                 close(fd);
 		fname = ggv_quote_filename (filename);
-		cmd = g_strdup_printf (gtk_gs_defaults_get_convert_pdf_cmd(),
+		cmd = g_strdup_printf (gtk_gs_defaults_get_dsc_cmd(),
                                        filename_dsc, fname);
 		g_free (fname);
 		/* this command (sometimes?) prints error messages to stdout! */
 		cmdline = g_strdup_printf ("%s >%s 2>&1", cmd, filename_err);
 		g_free (cmd);
 
-		if ((system (cmdline) == 0)
-		    && ggv_file_readable (filename_dsc) ) {
+                g_message("cmdline: %s", cmdline);
+
+		if ((system (cmdline) == 0) &&
+                    ggv_file_readable (filename_dsc) ) {
 
 			/* success */
 			filename = gs->gs_filename_dsc = filename_dsc;
@@ -1641,8 +1602,12 @@ gtk_gs_new (GtkAdjustment *hadj, GtkAdjustment *vadj)
 {
         GtkGS *gs;
 
-        g_return_val_if_fail(hadj != NULL, NULL);
-        g_return_val_if_fail(vadj != NULL, NULL);
+        if(NULL == hadj)
+                hadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 1.0,
+                                                         0.01, 0.1, 0.09));
+        if(NULL == vadj)
+                vadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 1.0,
+                                                         0.01, 0.1, 0.09));
 
 	gs = (GtkGS *)gtk_type_new(gtk_gs_get_type());
 
@@ -1744,8 +1709,6 @@ gtk_gs_disable_interpreter (GtkGS *gs)
 gboolean
 gtk_gs_load (GtkGS *gs, const gchar *fname)
 {
-        struct stat sbuf;
-
         g_return_val_if_fail(gs != NULL, FALSE);
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
@@ -1759,6 +1722,8 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 		return FALSE;
 	}
 
+        g_message("File: %s\n", fname);
+
  	/* prepare this document */
 
  	/* default values: no dsc information available  */
@@ -1842,20 +1807,11 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
                         /* Otherwise, set the proper orientation for the doc */
                         gs->real_orientation = gs->doc->orientation;
                 }
-                stat(fname, &sbuf);
-                gs->mtime = sbuf.st_mtime;
   	}
         gtk_gs_set_page_size(gs, -1, gs->current_page);
 	gtk_widget_queue_resize (&(gs->widget));
         gs->loaded = TRUE;
 
-	if(gs->watch_doc) {
-		if(gs->timer_tag)
-                        gtk_timeout_remove(gs->timer_tag);
-		gs->timer_tag = gtk_timeout_add(GTK_GS_WATCH_INTERVAL,
-                                                timer_callback, gs);
-	}
-
         gs->gs_status = _("Document loaded.");
 
 	return gs->loaded;
@@ -1931,14 +1887,6 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 return FALSE;
         }
 
-        if (gs->gs_psfile) {
-                /* has it changed? */
-                if (!stat(gs->gs_filename, &sbuf) &&
-                    gs->mtime != sbuf.st_mtime) {
-                        gtk_signal_emit(GTK_OBJECT(gs), gtk_gs_signals[DOCUMENT_CHANGED]);
-                }
-        }
-
         /* range checking... */
         if (page < 0)
                 page = 0;
@@ -2196,9 +2144,8 @@ gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
                 zoom = ggv_zoom_levels[ggv_max_zoom_levels];
         if (fabs(gs->zoom_factor - zoom) > 0.001) { 
                 gs->zoom_factor = zoom;
-                if (GTK_WIDGET_REALIZED(gs)) {
+                if (GTK_WIDGET_REALIZED(gs)) 
                         set_up_page(gs);
-                }
                 gs->changed = TRUE;
                 gtk_widget_queue_resize (&(gs->widget));
         }
@@ -2361,33 +2308,6 @@ gtk_gs_get_antialiasing(GtkGS *gs)
         return gs->antialiased;
 }
 
-void
-gtk_gs_set_watch_file(GtkGS *gs, gboolean f)
-{
-        g_return_if_fail(gs != NULL);
-        g_return_if_fail(GTK_IS_GS(gs));
-
-        if(gs->watch_doc == f)
-                return;
-
-        gs->watch_doc = f;
-	if(gs->watch_doc) {
-		if(gs->timer_tag)
-                        gtk_timeout_remove(gs->timer_tag);
-		gs->timer_tag = gtk_timeout_add(GTK_GS_WATCH_INTERVAL,
-                                                timer_callback, gs);
-	}
-}
-
-gint
-gtk_gs_get_watch_file(GtkGS *gs)
-{
-        g_return_val_if_fail(gs != NULL, -1);
-        g_return_val_if_fail(GTK_IS_GS(gs), -1);
-
-        return gs->watch_doc;
-}
-
 const gchar *
 gtk_gs_get_document_title(GtkGS *gs)
 {
@@ -2675,15 +2595,34 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         sink = gtk_gs_doc_sink_new();
 
         if(GTK_GS_IS_PDF(gs)) {
-                /* Save marked pages of the dsc file. The saved
-                 * file is useless without the pdf file, but can
-                 * be used to print the marked pages.
-                 * Printing only works with gs, not with a 'real'
-                 * postscript printer.
-                 * FIXME: Can we produce a real pdf file with ps2pdf?
-                 */
-                pscopydoc(sink, gs->gs_filename_dsc,
-                          gs->doc, pages);
+                gchar *tmpn = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+                gchar *cmd, *fname;
+                int tmpfd;
+
+                if((tmpfd = mkstemp(tmpn)) < 0) {
+                        g_free(tmpn);
+                        return NULL;
+                }
+                close(tmpfd);
+                fname = ggv_quote_filename(gs->gs_filename_unc?
+                                           gs->gs_filename_unc:
+                                           gs->gs_filename);
+                cmd = g_strdup_printf(gtk_gs_defaults_get_convert_pdf_cmd(),
+                                      tmpn, fname);
+                g_message("Cmdline 2 %s", cmd);
+                g_free(fname);
+                if((system(cmd) == 0) && ggv_file_readable(tmpn)) {
+                        GtkWidget *tmp_gs;
+                        tmp_gs = gtk_gs_new_from_file(NULL, NULL, tmpn);
+                        if(NULL != tmp_gs) {
+                                if(GTK_GS(tmp_gs)->loaded)
+                                        pscopydoc(sink, tmpn,
+                                                  GTK_GS(tmp_gs)->doc, pages);
+                                gtk_widget_destroy(tmp_gs);
+                        }
+                }
+                g_free(cmd);
+                g_free(tmpn);
         }
         else {
                 /* Use uncompressed file if necessary */
@@ -2691,7 +2630,6 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         }
         if(free_pages)
                 g_free(pages);
-
         doc = gtk_gs_doc_sink_get_buffer(sink);
         gtk_gs_doc_sink_free(sink);
         return doc;
